//+------------------------------------------------------------------+
//|                                        AdaptiveLaguerre_v2.1.mq4 |
//|                                Copyright  2008, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
// List of Prices:
// Price    = 0 - Close  
// Price    = 1 - Open  
// Price    = 2 - High  
// Price    = 3 - Low  
// Price    = 4 - Median Price   = (High+Low)/2  
// Price    = 5 - Typical Price  = (High+Low+Close)/3  
// Price    = 6 - Weighted Close = (High+Low+Close*2)/4
// Price    = 7 - Heiken Ashi Close  
// Price    = 8 - Heiken Ashi Open
// Price    = 9 - Heiken Ashi High
// Price    =10 - Heiken Ashi Low

// List of MAs for Gamma Smoothing:
// MA_Method= 0: SMA        - Simple Moving Average
// MA_Method= 1: EMA        - Exponential Moving Average
// MA_Method= 2: Wilder     - Wilder Exponential Moving Average
// MA_Method= 3: LWMA       - Linear Weighted Moving Average 
// MA_Method= 4: SineWMA    - Sine Weighted Moving Average
// MA_Method= 5: TriMA      - Triangular Moving Average
// MA_Method= 6: LSMA       - Least Square Moving Average (or EPMA, Linear Regression Line)
// MA_Method= 7: SMMA       - Smoothed Moving Average
// MA_Method= 8: HMA        - Hull Moving Average by Alan Hull
// MA_Method= 9: ZeroLagEMA - Zero-Lag Exponential Moving Average
// MA_Method=10: DEMA       - Double Exponential Moving Average by Patrick Mulloy
// MA_Method=11: T3         - T3 by T.Tillson
// MA_Method=12: ITrend     - Instantaneous Trendline by J.Ehlers
// MA_Method=13: Median     - Moving Median
// MA_Method=14: GeoMean    - Geometric Mean
// MA_Method=15: REMA       - Regularized EMA by Chris Satchwell
// MA_Method=16: ILRS       - Integral of Linear Regression Slope 
// MA_Method=17: IE/2       - Combination of LSMA and ILRS 
// MA_Method=18: TriMAgen   - Triangular Moving Average generalized by J.Ehlers

 
#property copyright "Copyright  2008, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_chart_window
#property indicator_buffers 5
#property indicator_color1  Linen
#property indicator_width1  1  
#property indicator_color2  SkyBlue
#property indicator_width2  1
#property indicator_color3  Tomato
#property indicator_width3  1
#property indicator_color4  SkyBlue
#property indicator_width4  2
#property indicator_color5  Tomato
#property indicator_width5  2 
//---- 
extern int     TimeFrame         =   0;   //Time Frame in min
extern int     Price             =   0;   //Price Mode (0...10)
extern int     L_Period          =   3;   //Laguerre Period(or Order,max=10)
extern int     AdaptLength       =  20;   //Adaptive Factor Length
extern double  Gamma             = 0.8;   //Laguerre Factor(GammaMode=0)
extern int     GammaMode         =   2;   //Gamma Mode: 0-non-adaptive,1-kaufman method,2-Ehlers method 
extern int     GammaSmooth       =   5;   //Gamma Smoothing Period
extern int     GammaSmoothMode   =   5;   //Gamma Smoothing Mode (0...19)
extern int     L_Shift           =   0;   //Shift in bars
extern double  PctFilter         = 0.5;   //Percent Filter
extern int     ColorMode         =   1;   //Switch of Color mode (1-color)
extern int     SignalMode        =   2;   //Switch of Signal mode(0-off,1-on)
extern int     AlertMode         =   0;   //Sound Alert switch(0...2) 
extern int     WarningMode       =   0;   //Warning Mode (0-off,1-on)
//---- 
double Laguerre[];
double UpTrend[];
double DnTrend[];
double UpSignal[];
double DnSignal[];
//----
double tmp[][11];
double L[][10];
double haClose[], haOpen[], haHigh[], haLow[];
double lastLo, lastHi;
int    draw_begin, pBars, mcnt_bars; 
string short_name;
datetime pTime;
bool   UpTrendAlert=false, DnTrendAlert=false;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
//---- 
   SetIndexStyle(0,DRAW_LINE);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexStyle(2,DRAW_LINE);
   SetIndexStyle(3,DRAW_ARROW);
   SetIndexArrow(3,159);
   SetIndexArrow(4,159);
   SetIndexStyle(4,DRAW_ARROW);
   
   if(TimeFrame == 0 || TimeFrame < Period()) TimeFrame = Period();
   SetIndexShift(0,L_Shift*TimeFrame/Period());
   SetIndexShift(1,L_Shift*TimeFrame/Period());
   SetIndexShift(2,L_Shift*TimeFrame/Period());
   SetIndexShift(3,L_Shift*TimeFrame/Period());
   SetIndexShift(4,L_Shift*TimeFrame/Period());
   
   draw_begin=2*L_Period*TimeFrame/Period();
//---- 
   switch(TimeFrame)
   {
   case 1     : string TF = "M1"; break;
   case 5     : TF = "M5"; break;
   case 15    : TF = "M15"; break;
   case 30    : TF = "M30"; break;
   case 60    : TF = "H1"; break;
   case 240   : TF ="H4"; break;
   case 1440  : TF="D1"; break;
   case 10080 : TF="W1"; break;
   case 43200 : TF="MN1"; break;
   default    : TF="Current";
   } 
   short_name = "AdaptiveLaguerre";
   IndicatorShortName(short_name+"("+L_Period+")"+" "+TF);
   SetIndexLabel(0,short_name+"("+L_Period+")"+" "+TF);
   SetIndexLabel(1,"UpTrend");
   SetIndexLabel(2,"DnTrend");
   SetIndexLabel(3,"UpSignal");
   SetIndexLabel(4,"DnSignal");
   
   SetIndexDrawBegin(0,draw_begin);
   SetIndexDrawBegin(1,draw_begin);
   SetIndexDrawBegin(2,draw_begin);
   SetIndexDrawBegin(3,draw_begin);
   SetIndexDrawBegin(4,draw_begin);
//---- 
   SetIndexBuffer(0,Laguerre);
   SetIndexBuffer(1,UpTrend);   
   SetIndexBuffer(2,DnTrend);
   SetIndexBuffer(3,UpSignal);
   SetIndexBuffer(4,DnSignal);
//---- 
   return(0);
}
//+------------------------------------------------------------------+
//| AdaptiveLaguerre_v2.1                                            |
//+------------------------------------------------------------------+
int start()
{
   int limit, y, i, shift, cnt_bars=IndicatorCounted(); 
   double price[], aPrice[], mL[], diff[], Gam[], gamma[], mUp[], mDn[], mUpsig[], mDnsig[], trend[];
   double lastLo[], lastHi[];
    
   if(TimeFrame!=Period()) int mBars = iBars(NULL,TimeFrame); else mBars = Bars;   
   
   if(mBars != pBars)
   {
   ArrayResize(price,mBars);
   ArrayResize(aPrice,mBars);
   ArrayResize(mL,mBars);
   ArrayResize(tmp,mBars);
   ArrayResize(L,mBars);
   ArrayResize(diff,mBars);
   ArrayResize(Gam,mBars);
   ArrayResize(gamma,mBars);
        
      if(Price > 6 && Price <= 10)
      {
      ArrayResize(haClose,mBars);
      ArrayResize(haOpen,mBars);
      ArrayResize(haHigh,mBars);
      ArrayResize(haLow,mBars);
      }
  
      if(ColorMode>0)
      {
      ArrayResize(trend,mBars);
      ArrayResize(mUp,mBars);
      ArrayResize(mDn,mBars);
      ArrayResize(mUpsig,mBars);
      ArrayResize(mDnsig,mBars);
      ArrayResize(lastLo,mBars); 
      ArrayResize(lastHi,mBars); 
      }
   pBars = mBars;
   }  
   
   if(cnt_bars<1)
   {
      for(i=Bars-1;i>0;i--) 
      {
      Laguerre[i]=EMPTY_VALUE; 
      UpTrend[i]=EMPTY_VALUE;
      DnTrend[i]=EMPTY_VALUE;
      UpSignal[i]=EMPTY_VALUE;
      DnSignal[i]=EMPTY_VALUE;
      }
   mcnt_bars = 0;
   }
//---- 
   if(mcnt_bars > 0) mcnt_bars--;
   
   for(y=mcnt_bars;y<mBars;y++)
   {
      
      if(y >= L_Period) 
      {
         if(Price <= 6) price[y] = iMA(NULL,TimeFrame,1,0,0,Price,mBars-y-1);   
         else
         if(Price > 6 && Price <= 10) price[y] = HeikenAshi(TimeFrame,Price-7,mBars-y-1);
              
         diff[y] = MathAbs(price[y] - mL[y-1]);
        
         if(GammaMode > 0)
         { 
         if(GammaMode == 1) Gam[y] = AdaptiveGamma(GammaMode,price,AdaptLength,y);
         if(GammaMode == 2) Gam[y] = AdaptiveGamma(GammaMode,diff,AdaptLength,y);
         
            switch(GammaSmoothMode)
            {
            case 0 : gamma[y] = SMA(Gam,GammaSmooth,y); break;
            case 1 : gamma[y] = EMA(Gam[y],gamma,GammaSmooth,y); break;
            case 2 : gamma[y] = Wilder(Gam,gamma,GammaSmooth,y); break;  
            case 3 : gamma[y] = LWMA(Gam,GammaSmooth,y); break;
            case 4 : gamma[y] = SineWMA(Gam,GammaSmooth,y); break;
            case 5 : gamma[y] = TriMA(Gam,GammaSmooth,y); break;
            case 6 : gamma[y] = LSMA(Gam,GammaSmooth,y); break;
            case 7 : gamma[y] = SMMA(Gam,gamma,GammaSmooth,y); break;
            case 8 : gamma[y] = HMA(Gam,GammaSmooth,y); break;
            case 9 : gamma[y] = ZeroLagEMA(Gam,gamma,GammaSmooth,y); break;
            case 10: gamma[y] = DEMA(0,Gam[y],GammaSmooth,1,y); break;
            case 11: gamma[y] = T3(Gam[y],GammaSmooth,0.7,y); break;
            case 12: gamma[y] = ITrend(Gam,gamma,GammaSmooth,y); break;
            case 13: gamma[y] = Median(Gam,GammaSmooth,y); break;
            case 14: gamma[y] = GeoMean(Gam,GammaSmooth,y); break;
            case 15: gamma[y] = REMA(Gam[y],gamma,GammaSmooth,0.5,y); break;
            case 16: gamma[y] = ILRS(Gam,GammaSmooth,y); break;
            case 17: gamma[y] = IE2(Gam,GammaSmooth,y); break;
            case 18: gamma[y] = TriMA_gen(Gam,GammaSmooth,y); break;
            case 19: gamma[y] = JSmooth(Gam[y],GammaSmooth,1,0,y); break;
            }
         }
         else
         gamma[y] = Gamma;
         
         for(i=0;i<L_Period;i++)
         {
            if(y == L_Period) L[y][i] = price[y];
            else
            {
               if(i == 0) L[y][i] = (1 - gamma[y])*price[y] + gamma[y]*L[y-1][i];
               else
               L[y][i] = -gamma[y]*L[y][i-1] + L[y-1][i-1] + gamma[y]*L[y-1][i];
            aPrice[y-i] = L[y][i];
            }
         }
         mL[y] = TriMA_gen(aPrice,L_Period,y);  
      
         if(ColorMode > 0)
         {
            if (PctFilter>0)
            {
            double sumdel=0;
            for (i=0;i<AdaptLength;i++) sumdel = sumdel+MathAbs(mL[y-i] - mL[y-i-1]);
            double AvgDel = sumdel/AdaptLength;
    
            double sumpow = 0;
            for (i=0;i<AdaptLength;i++) sumpow+=MathPow(MathAbs(mL[y-i] - mL[y-i-1])-AvgDel,2);
            double StdDev = MathPow(sumpow/AdaptLength,0.5); 
     
            double Filter = PctFilter * StdDev;
            }
            else Filter = 0;
         
         if(y == L_Period+1)
         {
         lastLo[y] = mL[y];
         lastHi[y] = mL[y];
         }
         
         lastLo[y]= lastLo[y-1];
         lastHi[y]= lastHi[y-1];
                  
         trend[y] = trend[y-1];
                 
         if (mL[y] > mL[y-1] && mL[y] - lastLo[y] > Filter && trend[y-1]<=0 && lastLo[y] >-100000) trend[y]= 1;
         if (mL[y] < mL[y-1] && lastHi[y] - mL[y] > Filter && trend[y-1]>=0 && lastHi[y] < 100000) trend[y]=-1; 
            
            if (trend[y]>0)
            {  
            lastHi[y] = MathMax(mL[y],lastHi[y-1]);
               if (mL[y] > mL[y-1])
               {
               mUp[y] = mL[y];
               mDn[y] = EMPTY_VALUE;
                  if (trend[y-1] != trend[y])
                  { 
                  lastLo[y] = 100000;
                  if (SignalMode == 1) mUpsig[y] = mL[y];  
                  else
                  if (SignalMode == 2) mUpsig[y] = lastLo[y-1];
                  if (WarningMode > 0 && y==mBars-1) PlaySound("alert2.wav");
                  }
                  else mUpsig[y] = EMPTY_VALUE;
               }
               else
               {
               mUp[y] = EMPTY_VALUE;
               mDn[y] = EMPTY_VALUE;
               mUpsig[y]= EMPTY_VALUE; 
               mDnsig[y]= EMPTY_VALUE;
               }
            }
            else
            if (trend[y]<0) 
            {
            lastLo[y] = MathMin(mL[y],lastLo[y-1]); 
               if (mL[y] < mL[y-1])
               {
               mDn[y] = mL[y];
               mUp[y] = EMPTY_VALUE;
                  if (trend[y-1] != trend[y])
                  { 
                  lastHi[y] =-100000; 
                  if (SignalMode == 1) mDnsig[y] = mL[y];  
                  else
                  if (SignalMode == 2) mDnsig[y] = lastHi[y-1]; 
                  if (WarningMode > 0 && y==mBars-1) PlaySound("alert2.wav");  
                  }
                  else mDnsig[y] = EMPTY_VALUE;
               }
               else
               {
               mUp[y] = EMPTY_VALUE;
               mDn[y] = EMPTY_VALUE;
               mUpsig[y]= EMPTY_VALUE; 
               mDnsig[y]= EMPTY_VALUE;
               }
            }
         //Comment("y=",y," aL=",mL[y]," trend=",trend[y]," lHi=",lastHi[y]," lLo=",lastLo[y]);
         }       
                       
         if(TimeFrame == Period()) 
         {
         Laguerre[mBars-y-1] = mL[y];
            if(ColorMode > 0)
            {
            UpTrend[mBars-y-1] = mUp[y];
            DnTrend[mBars-y-1] = mDn[y];
               if(SignalMode > 0)
               {
               UpSignal[mBars-y-1] = mUpsig[y];
               DnSignal[mBars-y-1] = mDnsig[y];
               }
            }   
         }
      }
      mcnt_bars = mBars-1;
   }
   
   if(TimeFrame > Period())
   { 
      if(cnt_bars>0) cnt_bars--;
      limit = Bars-cnt_bars+TimeFrame/Period()-1;
      
      for(shift=0,y=0;shift<limit;shift++)
      {
      if (Time[shift] < iTime(NULL,TimeFrame,y)) y++; 
      Laguerre[shift] = mL[mBars-y-1];
         if(ColorMode == 1)
         {
         UpTrend[shift] = mUp[mBars-y-1];
         DnTrend[shift] = mDn[mBars-y-1];
            if(SignalMode > 0)
            {
            UpSignal[shift] = mUpsig[mBars-y-1];
            DnSignal[shift] = mDnsig[mBars-y-1];
            }
         }
      }
   }
//----------   
   string Message;
   
   if (trend[mBars-2]>0 && mL[mBars-2] > mL[mBars-3] && ((!UpTrendAlert && AlertMode == 1) || (AlertMode == 2)))
	{
	Message = " "+Symbol()+" M"+Period()+":AdaptiveLaguerre Signal for BUY";
	if (isNewBar()) Alert (Message); 
	UpTrendAlert=true; DnTrendAlert=false;
	} 
	else 	  
	if (trend[mBars-2]<0 && mL[mBars-2] < mL[mBars-3] && ((!DnTrendAlert && AlertMode == 1) || (AlertMode == 2)))
	{
	Message = " "+Symbol()+" M"+Period()+":AdaptiveLaguerre Signal for SELL";
	if (isNewBar()) Alert (Message); 
	DnTrendAlert=true; UpTrendAlert=false;
	} 	         
   return(0);
}
// MA_Method=0: SMA - Simple Moving Average
double SMA(double array[],int per,int bar)
{
   double Sum = 0;
   for(int i = 0;i < per;i++) Sum += array[bar-i];
     
   return(Sum/per);
}                
// MA_Method=1: EMA - Exponential Moving Average
double EMA(double price,double array[],int per,int bar)
{
   if(bar == per) double ema = price;
   else 
   if(bar > per) ema = array[bar-1] + 2.0/(1+per)*(price - array[bar-1]); 
   
   return(ema);
}
// MA_Method=2: Wilder - Wilder Exponential Moving Average
double Wilder(double array1[],double array2[],int per,int bar)
{
   if(bar == per) double wilder = SMA(array1,per,bar);
   else 
   if(bar > per) wilder = array2[bar-1] + (array1[bar] - array2[bar-1])/per; 
   
   return(wilder);
}
// MA_Method=3: LWMA - Linear Weighted Moving Average 
double LWMA(double array[],int per,int bar)
{
   double Sum = 0;
   double Weight = 0;
   
      for(int i = 0;i < per;i++)
      { 
      Weight+= (per - i);
      Sum += array[bar-i]*(per - i);
      }
   if(Weight>0) double lwma = Sum/Weight;
   else lwma = 0; 
   return(lwma);
} 
// MA_Method=4: SineWMA - Sine Weighted Moving Average
double SineWMA(double array[],int per,int bar)
{
   double pi = 3.1415926535;
   double Sum = 0;
   double Weight = 0;
  
      for(int i = 0;i < per;i++)
      { 
      Weight+= MathSin(pi*(i+1)/(per+1));
      Sum += array[bar-i]*MathSin(pi*(i+1)/(per+1)); 
      }
   if(Weight>0) double swma = Sum/Weight;
   else swma = 0; 
   return(swma);
}
// MA_Method=5: TriMA - Triangular Moving Average
double TriMA(double array[],int per,int bar)
{
   int len1 = MathCeil(per*0.5);
   int len2 = MathFloor(per*0.5);
   
   double sum = 0;
   double weight = 0;
   double sumw=0;
   for(int i = 0;i < per;i++) 
   {
   if(i<=len1-1) weight+=1;
   if(i==len1 && len1 == len2) weight = weight;
   else if(i>len1-1) weight -= 1;  
   sum += array[bar-i]*weight;
   sumw += weight; 
   } 
   double trima = sum/sumw;
   
   return(trima);
}
// MA_Method=6: LSMA - Least Square Moving Average (or EPMA, Linear Regression Line)
double LSMA(double array[],int per,int bar)
{   
   double Sum=0;
   for(int i=per; i>=1; i--) Sum += (i-(per+1)/3.0)*array[bar-per+i];
   double lsma = Sum*6/(per*(per+1));
   return(lsma);
}
// MA_Method=7: SMMA - Smoothed Moving Average
double SMMA(double array1[],double array2[],int per,int bar)
{
   if(bar == per) double smma = SMA(array1,per,bar);
   else 
   if(bar > per)
   {
   double Sum = 0;
   for(int i = 0;i < per;i++) Sum += array1[bar-i-1];
   smma = (Sum - array2[bar-1] + array1[bar])/per;
   }
   return(smma);
}                
// MA_Method=8: HMA - Hull Moving Average by Alan Hull
double HMA(double array[],int per,int bar)
{
   double tmp[];
   int len =  MathSqrt(per);
   ArrayResize(tmp,len);
   
   if(bar == per) double hma = array[bar]; 
   else
   if(bar > per)
   {
   for(int i = 0; i < len;i++) tmp[len-i-1] = 2*LWMA(array,per/2,bar-i) - LWMA(array,per,bar-i);  
   hma = LWMA(tmp,len,len-1); 
   }  

   return(hma);
}
// MA_Method=9: ZeroLagEMA - Zero-Lag Exponential Moving Average
double ZeroLagEMA(double array1[],double array2[],int per,int bar)
{
   double alfa = 2.0/(1+per); 
   int lag = 0.5*(per - 1); 
   
   if(bar == lag) double zema = array1[bar];
   else 
   if(bar > lag) zema = alfa*(2*array1[bar] - array1[bar-lag]) + (1-alfa)*array2[bar-1];
   
   return(zema);
}
// MA_Method=10: DEMA - Double Exponential Moving Average by Patrick Mulloy
double DEMA(int num,double price,int per,double v,int bar)
{
   if(bar == 2) {double dema = price; tmp[bar][num] = dema; tmp[bar][num+1] = dema;}
   else 
   if(bar > 2) 
   {
   tmp[bar][num] = tmp[bar-1][num] + 2.0/(1+per)*(price - tmp[bar-1][num]); 
   tmp[bar][num+1] = tmp[bar-1][num+1] + 2.0/(1+per)*(tmp[bar][num] - tmp[bar-1][num+1]); 
   dema = (1+v)*tmp[bar][num] - v*tmp[bar][num+1];
   }
   return(dema);
}
// MA_Method=11: T3 by T.Tillson
double T3(double price,int per,double v,int bar)
{
   if(bar == 2) 
   {
   double T3 = price; 
   for(int k=0;k<=5;k++) tmp[bar][k] = T3;
   }
   else 
   if(bar > 2) 
   {
   double dema1 = DEMA(0,price,per,v,bar); 
   double dema2 = DEMA(2,dema1,per,v,bar); 
   T3 = DEMA(4,dema2,per,v,bar);
   }
   return(T3);
}
// MA_Method=12: ITrend - Instantaneous Trendline by J.Ehlers
double ITrend(double price[],double array[],int per,int bar)
{
   double alfa = 2.0/(per+1);
   if(bar > 7)
   double it = (alfa - alfa*alfa/4)*price[bar]+ 0.5*alfa*alfa*price[bar-1]-(alfa - 0.75*alfa*alfa)*price[bar-2]+
   2*(1-alfa)*array[bar-1] - (1-alfa)*(1-alfa)*array[bar-2];
   else
   it = (price[bar] + 2*price[bar-1]+ price[bar-2])/4;
   
   return(it);
}
// MA_Method=13: Median - Moving Median
double Median(double price[],int per,int bar)
{
   double array[];
   ArrayResize(array,per);
   
   for(int i = 0; i < per;i++) array[i] = price[bar-i];
   ArraySort(array);
   
   int num = MathRound((per-1)/2); 
   if(MathMod(per,2)>0) double median = array[num]; else median = 0.5*(array[num]+array[num+1]);
   
   return(median); 
}
// MA_Method=14: GeoMean - Geometric Mean
double GeoMean(double price[],int per,int bar)
{
   double gmean = MathPow(price[bar],1.0/per); 
   for(int i = 1; i < per;i++) gmean *= MathPow(price[bar-i],1.0/per); 
   
   return(gmean);
}
// MA_Method=15: REMA - Regularized EMA by Chris Satchwell 
double REMA(double price,double array[],int per,double lambda,int bar)
{
   double alpha =  2.0/(per + 1);
   if(bar <= 3) double rema = price;
   else 
   if(bar > 3) 
   rema = (array[bar-1]*(1+2*lambda) + alpha*(price - array[bar-1]) - lambda*array[bar-2])/(1+lambda); 
   
   return(rema);
}
// MA_Method=16: ILRS - Integral of Linear Regression Slope 
double ILRS(double price[],int per,int bar)
{
   double sum = per*(per-1)*0.5;
   double sum2 = (per-1)*per*(2*per-1)/6.0;
     
   double sum1 = 0;
   double sumy = 0;
      for(int i=0;i<per;i++)
      { 
      sum1 += i*price[bar-i];
      sumy += price[bar-i];
      }
   double num1 = per*sum1 - sum*sumy;
   double num2 = sum*sum - per*sum2;
   
   if(num2 != 0) double slope = num1/num2; else slope = 0; 
   double ilrs = slope + SMA(price,per,bar);
   
   return(ilrs);
}
// MA_Method=17: IE/2 - Combination of LSMA and ILRS 
double IE2(double price[],int per,int bar)
{
   double ie = 0.5*(ILRS(price,per,bar) + LSMA(price,per,bar));
      
   return(ie); 
}
 
// MA_Method=18: TriMAgen - Triangular Moving Average Generalized by J.Ehlers
double TriMA_gen(double array[],int per,int bar)
{
   int len1 = MathFloor((per+1)*0.5);
   int len2 = MathCeil((per+1)*0.5);
   double sum=0;
   for(int i = 0;i < len2;i++) sum += SMA(array,len1,bar-i);
   double trimagen = sum/len2;
   
   return(trimagen);
}

// MA_Method=19: JSmooth - Smoothing by Mark Jurik
double JSmooth(double price,int per,double pow,int num,int bar)
{
   double beta = 0.45*(per-1)/(0.45*(per-1)+2);
	double alpha = MathPow(beta,pow);
	if(bar == 2) {tmp[bar][4] = price; tmp[bar][0] = price; tmp[bar][2] = price;}
	else 
   if(bar > 2) 
   {
	tmp[bar][num]   = (1-alpha)*price + alpha*tmp[bar-1][num];
	tmp[bar][num+1] = (price - tmp[bar][num])*(1-alpha) + alpha*tmp[bar-1][num+1];
	tmp[bar][num+2] = tmp[bar][num] + tmp[bar][num+1];
	tmp[bar][num+3] = (tmp[bar][num+2] - tmp[bar-1][num+4])*MathPow((1-alpha),2) + MathPow(alpha,2)*tmp[bar-1][num+3];
	tmp[bar][num+4] = tmp[bar-1][num+4] + tmp[bar][num+3]; 
   }
   return(tmp[bar][num+4]);
}

double HeikenAshi(int tf,int price,int bar)
{ 
   if(bar == iBars(NULL,TimeFrame)- 1) 
   {
   haClose[bar] = iClose(NULL,tf,bar);
   haOpen[bar]  = iOpen(NULL,tf,bar);
   haHigh[bar]  = iHigh(NULL,tf,bar);
   haLow[bar]   = iLow(NULL,tf,bar);
   }
   else
   {
   haClose[bar] = (iOpen(NULL,tf,bar)+iHigh(NULL,tf,bar)+iLow(NULL,tf,bar)+iClose(NULL,tf,bar))/4;
   haOpen[bar]  = (haOpen[bar+1]+haClose[bar+1])/2;
   haHigh[bar]  = MathMax(iHigh(NULL,tf,bar),MathMax(haOpen[bar], haClose[bar]));
   haLow[bar]   = MathMin(iLow(NULL,tf,bar),MathMin(haOpen[bar], haClose[bar]));
   }
   
   switch(price)
   {
   case 0: return(haClose[bar]);break;
   case 1: return(haOpen[bar]);break;
   case 2: return(haHigh[bar]);break;
   case 3: return(haLow[bar]);break;
   }
}     
   
double AdaptiveGamma(int mode,double array[],int per,int bar)
{
   double sum = 0;
   double max = 0, min = 1000000000; 
   for(int i = 0;i < per;i++) 
   {
      if(mode == 1) sum += MathAbs(array[bar-i] - array[bar-i-1]); 
      else
      if(mode == 2)
      {
      if (array[bar-i] > max) max = array[bar-i]; 
      if (array[bar-i] < min) min = array[bar-i]; 
      }
   }
   
   if(mode == 1 && sum!= 0) double eff = MathAbs(array[bar] - array[bar-per])/sum;
   else 
   if(mode == 2 && max-min != 0) eff = (array[bar] - min)/(max - min);
   else
   eff = 0;
   return(1 - eff);  
} 

bool isNewBar()
{
   bool res=false;
   if (iTime(NULL,TimeFrame,0)!=pTime)
   {
   res=true;
   pTime=iTime(NULL,TimeFrame,0);
   }   
   return(res);
}        		